local L = LibStub("AceLocale-3.0"):GetLocale("Chinon")
local HealComm = LibStub("LibHealComm-4.0")

-- Un bouton servant aux soins

local APortee_Margin = 2
local BarreVie_Margin = 2

local READY_CHECK_WAITING_TEXTURE = "Interface\\RaidFrame\\ReadyCheck-Waiting";
local READY_CHECK_READY_TEXTURE = "Interface\\RaidFrame\\ReadyCheck-Ready";
local READY_CHECK_NOT_READY_TEXTURE = "Interface\\RaidFrame\\ReadyCheck-NotReady";


function Chinon_Bouton_SetTexte(self, pos)
	self.prio:SetText(pos);
end

function Chinon_Bouton_AddBuff(self, icon, count, expirationTime)
	self.nbBuffs=self.nbBuffs+1
	if self.nbBuffs>#self.buff then
		self:ConfigureBuff(self.nbBuffs)
	end
	self.buff[self.nbBuffs]:Show();
	self.buff[self.nbBuffs].texture:SetTexture(icon);
	local remains = expirationTime-GetTime()
	if (count==0) then
		count = math.floor(remains)
	end
	self.buff[self.nbBuffs].stacks:SetText(count);
	if (remains<1.5) then
		self.buff[self.nbBuffs].stacks:SetTextColor(1,0,0);
	elseif (remains<=3) then
		self.buff[self.nbBuffs].stacks:SetTextColor(1,0.5,0);
	else
		self.buff[self.nbBuffs].stacks:SetTextColor(1,1,1);
	end
end

function Chinon_Bouton_HideInfo(self)
	if (not self.hidden) then
		if (self.mana) then
			self.mana:Hide()
		end
		self.vie:Hide();
		self.viePerdue:Hide()
		self.barreSoin:Hide();
		self.barreDegats:Hide();
		for i,aPortee in pairs(self.aPortee) do
			aPortee:Hide();
		end
		
		for i,debuff in pairs(self.debuff) do		
			debuff:Hide();
		end
		self.hidden = true
	end
end

function Chinon_Bouton_OnUpdate(self)
	local unit=self:GetAttribute("unit");
	local cfgBouton = self.cfgBouton;
	
	if (cfgBouton == nil) then
		return;
	end
	
	if (not UnitExists(unit)) then
		Chinon_Bouton_HideInfo(self)
		return
	end
	
	if (Chinon.unitResTarget[unit] and Chinon.unitResTarget[unit]>0) then
		if (UnitIsDeadOrGhost(unit)) then
			Chinon_Bouton_HideInfo(self)
			self.barreSoin:Show()
			self.barreSoin:SetWidth(self:GetWidth()-BarreVie_Margin*2)
			return
		end
		Chinon.unitResTarget[unit] = 0
	end
	
	if (UnitIsDeadOrGhost(unit)) then
		Chinon_Bouton_HideInfo(self)
		return
	end
	
	self.hidden = false
		
	if (cfgBouton.mana and UnitPowerMax(unit) and UnitPowerMax(unit)>0) then
		self.mana:Show()
		local rapport = UnitPower(unit) / UnitPowerMax(unit) 
		if (rapport>1) then
			rapport=1;
		end
		self.mana:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2));
	elseif (self.mana) then
		self.mana:Hide()
	end
	
	
	if (cfgBouton.vie and UnitHealthMax(unit) and UnitHealthMax(unit)>0) then
		self.vie:Show();
		self.viePerdue:Show()
		local rapport=UnitHealth(unit)/UnitHealthMax(unit);
		local blue = 0.5
		if (cfgBouton.aggro) and (UnitThreatSituation(unit) and UnitThreatSituation(unit)>1) then
			blue = 0
		end
	
		if (rapport>0.9) then
			self.vie:SetGradient("HORIZONTAL", 0,0.8,blue, 0,0.5,blue)
		elseif (rapport>0.5) then
			self.vie:SetGradient("HORIZONTAL", 0.5,1,blue, 0.5,0.75,blue);
		elseif (rapport>0.1) then
			self.vie:SetGradient("HORIZONTAL", 0.65,0.2,blue, 0.45,0,blue);
		else
			self.vie:SetGradient("HORIZONTAL", 1,0,blue, 0.8,0,blue);
		end
		if (rapport>1) then
			rapport=1;
		end
		self.vie:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2));
		if (rapport<1) then
			self.viePerdue:SetWidth((1-rapport)*(self:GetWidth()-BarreVie_Margin*2));
		else
			self.viePerdue:Hide()
		end

		local soin = HealComm:GetHealAmount(UnitGUID(unit), HealComm.CASTED_HEALS, GetTime()+2) or 0
		if (soin>0) then
			local rapport = soin/UnitHealthMax(unit)
			if (rapport>1) then
				rapport=1
			end
			
			self.barreSoin:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2))
			self.barreSoin:Show()
		else
			self.barreSoin:Hide();
		end
		
		if (Chinon_Frame.totalDegats[unit] and Chinon_Frame.totalDegats[unit]>0) then
			local rapport = Chinon_Frame.totalDegats[unit]/UnitHealthMax(unit)
			if (rapport>1) then
				rapport=1
			end
			self.barreDegats:SetWidth(rapport*(self:GetWidth()-BarreVie_Margin*2))
			self.barreDegats:Show()
		else
			self.barreDegats:Hide();
		end
	else
		self.viePerdue:Hide()
		self.vie:Hide();
		self.barreSoin:Hide();
		self.barreDegats:Hide()
	end

	-- portée
	
	for i, portee in pairs(self.aPortee) do
		portee:Hide();
	end
		
	for i, portee in pairs(cfgBouton.portee) do
		if (portee.sort and IsSpellInRange(GetSpellInfo(portee.sort),unit)==1) then
			self.aPortee[i]:Show();
		end
	end
	
	-- Buffs
	if self.nbBuffs then
		for i=1,self.nbBuffs do
			self.buff[i]:Hide()
		end
	end
	
	self.nbBuffs=0
	
	if (#cfgBouton.buff) then
		local i=1;
		while 1 do
			local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable =  UnitBuff(unit, i, true);
			if (not name) then
				break;
			end
			if (cfgBouton.sortBuff[name] and (unitCaster=="player" or cfgBouton.sortBuff[name].tous) and
				(not Chinon_Frame.enCombat or not cfgBouton.sortBuff[name].horsCombat)) then
				self:AddBuff(icon, count, expirationTime)
			end
			i=i+1;
		end
	end
	
	-- Texte
	if (cfgBouton.pv or cfgBouton.menace) then
		local text = ""
		if (cfgBouton.nomCible and GetNumRaidMembers()>5) then
			text = UnitName(unit)
		end
		if (cfgBouton.pv) then
			local health = UnitHealth(unit)
			if (health<UnitHealthMax(unit)) then
				if (text~="") then
					text = text .. " "
				end
				text = text .. health
			end
		end
		if (cfgBouton.menace) then
			local  _,_,threatpct,_,_ = UnitDetailedThreatSituation(unit, unit.."target")
			if (threatpct and threatpct>75) then
				if (text~="") then
					text = text .. " "
				end
				text = text .. string.format("%d%%",threatpct)
			end
		end
		self.prio:SetText(text)
	end

	-- Debuffs	
	if (#cfgBouton.debuff) then
		for i, debuff in pairs(cfgBouton.debuff) do
			self.isDebuff[i] = false;
			self.debufLevel[i] = 0;
		end
		
		local i=1;
		while 1 do
			local debuffName, debuffRank, debuffTexture, debuffApplications, debuffType, duration, expirationTime =UnitDebuff(unit,i);
			if (not debuffName) then
				break;
			end
			
			if (Chinon.warning[debuffName]) then
				for i, debuff in pairs(cfgBouton.debuff) do
					self.isDebuff[i] = true;
					self.debufLevel[i] = 4;
				end
			end
			
			local debuff2 = cfgBouton.sortDebuff2[debuffName]
			if debuff2~= nil and (not debuff2.minStacks or debuff2.minStacks>=debuffApplications) then
				self:AddBuff(debuffTexture, debuffApplications, expirationTime)
			end
			
			local typeDebuff = 0;
			
			if (cfgBouton.sortDebuff[debuffName]~= nil) then
				typeDebuff = cfgBouton.sortDebuff[debuffName];
				self.isDebuff[typeDebuff] = true;
			end
				
			if (cfgBouton.sortDebuff[debuffType]~= nil) then
				typeDebuff = cfgBouton.sortDebuff[debuffType];
				self.isDebuff[typeDebuff] = true;
			end

			if (typeDebuff>0) then
				if (Chinon.decurseLevel[debuffName] and Chinon.decurseLevel[debuffName]>self.debufLevel[typeDebuff]) then
					self.debufLevel[typeDebuff] = Chinon.decurseLevel[debuffName];
				end

				if (Chinon.decurseLevel[debuffType] and Chinon.decurseLevel[debuffType]>self.debufLevel[typeDebuff]) then
					self.debufLevel[typeDebuff] = Chinon.decurseLevel[debuffType];
				end
			end
			
			i=i+1;
		end

		for i, debuff in pairs(self.debuff) do
			debuff:Hide()
		end
		
		for i, debuff in pairs(cfgBouton.debuff) do
			if (self.isDebuff[i]) then
				if (cfgBouton.porteeDebuff[i] and IsSpellInRange(cfgBouton.porteeDebuff[i],unit)==1) then
					if (self.debufLevel[i]==4) then
						self.debuff[i]:SetTexture(1.0,1.0,1.0);
					elseif (self.debufLevel[i]==3) then
						self.debuff[i]:SetTexture(1.0,0.0,0.0);
					elseif (self.debufLevel[i]==2) then
						self.debuff[i]:SetTexture(1.0,1.0,0.0);
					elseif (self.debufLevel[i]==1) then
						self.debuff[i]:SetTexture(0.0,0.6,0.0);
					else
						self.debuff[i]:SetTexture(0.2,0.2,1.0);
					end
				else
					self.debuff[i]:SetTexture(0,0,0);
				end
				self.debuff[i]:Show();
			end
		end
	end
	
	if cfgBouton.role then
		local texture = nil
		local status = GetReadyCheckStatus(unit)
		if status~=nil then
			if status == "notready" then
				texture = READY_CHECK_NOT_READY_TEXTURE
			elseif status == "ready" then
				texture = READY_CHECK_READY_TEXTURE
			elseif status == "waiting" then
				texture = READY_CHECK_WAITING_TEXTURE
			end
		elseif GetNumRaidMembers()>1 then
			if Chinon.role[unit] == "MAINTANK" then
				texture = "Interface\\GroupFrame\\UI-Group-MainTankIcon"
			elseif Chinon.role[unit] == "MAINASSIST" then
				texture = "Interface\\GroupFrame\\UI-Group-MainAssistIcon"
			end
		else
			if UnitGroupRolesAssigned(unit) then
				texture = "Interface\\GroupFrame\\UI-Group-MainTankIcon"
			end
		end
		if self.roleTexture~=texture then
			if texture then
				self.role:SetTexture(texture)
				self.role:Show()
			else
				self.role:Hide()
			end
			self.roleTexture=texture
		end
	end
end

function Chinon_Bouton_OnLoad(self)
end

function Chinon_Bouton_Init(self)
	self.Configure = Chinon_Bouton_Configure;
	self.OnUpdate = Chinon_Bouton_OnUpdate;
	self.SetTexte = Chinon_Bouton_SetTexte;
	self.AddSpell = Chinon_Bouton_AddSort;
	self.OnEnter = Chinon_Bouton_OnEnter;
	self.OnLeave = Chinon_Bouton_OnLeave;
	self.AddBuff = Chinon_Bouton_AddBuff
	self.ConfigureBuff = Chinon_Bouton_ConfigureBuff
	self.isDebuff = {}
	self.debufLevel = {}
end

function Chinon_Bouton_ConfigureBuff(self, i)
	local largeurBuff = Chinon.apparence.hauteurBouton / 2

	if (not self.buff[i]) then
		self.buff[i] = CreateFrame("Frame",nil,self,nil)
		self.buff[i].texture = self.buff[i]:CreateTexture()
		self.buff[i].stacks = self.buff[i]:CreateFontString(nil, "OVERLAY");
		self.buff[i].stacks:SetNonSpaceWrap(false)
	end
	
	self.buff[i]:SetPoint("BOTTOMRIGHT",-largeurBuff*(i-1),0);
	self.buff[i]:SetHeight(largeurBuff);
	self.buff[i]:SetWidth(largeurBuff);
	self.buff[i].texture:SetAllPoints(self.buff[i])
	
	self.buff[i].stacks:SetFontObject("GameFontNormal");
	self.buff[i].stacks:SetAllPoints(self.buff[i])
	self.buff[i].stacks:SetText("");
	self.buff[i].stacks:SetTextColor(1,1,1);
	self.buff[i]:Hide();
end

function Chinon_Bouton_Configure(self, cfgBouton)
	self.cfgBouton = cfgBouton;
    
	Chinon:AppliquerFrameSkin(self, Chinon.apparence.buttonSkin)
	self:SetBackdropBorderColor(1,1,1);
	
	self:SetWidth(Chinon.apparence.largeurBouton);
	self:SetHeight(Chinon.apparence.hauteurBouton);
	self:RegisterForClicks("AnyUp");

	local hauteurBarreVie = self:GetHeight()/2-BarreVie_Margin
	local hauteurBarreMana = self:GetHeight()/10
	if (cfgBouton.mana) then
		hauteurBarreVie= hauteurBarreVie - hauteurBarreMana
	end
	--La vie restante
	if (not self.vie) then
		self.vie = self:CreateTexture();
		self.vie:SetTexture(1,1,1);
		self.vie:SetDrawLayer("ARTWORK");
	end
	self.vie:SetPoint("TOPLEFT",BarreVie_Margin,-BarreVie_Margin);
	self.vie:SetWidth(self:GetWidth()-BarreVie_Margin*2)
	self.vie:SetHeight(hauteurBarreVie);
	
	--La vie perdue (permet de compléter la barre de vie)
	if (not self.viePerdue) then
		self.viePerdue = self:CreateTexture();
		self.viePerdue:SetTexture(0,0,0,0.5);
		self.viePerdue:SetDrawLayer("ARTWORK");
	end
	self.viePerdue:SetPoint("TOPRIGHT",-BarreVie_Margin,-BarreVie_Margin);
	self.viePerdue:SetWidth(0)
	self.viePerdue:SetHeight(hauteurBarreVie);
	
	-- La barre de mana
	if (cfgBouton.mana) then
		if (not self.mana) then
			self.mana = self:CreateTexture();
			self.mana:SetTexture(0,0,1);
			self.mana:SetDrawLayer("ARTWORK"); 
		end
		self.mana:SetPoint("TOPLEFT",BarreVie_Margin,-BarreVie_Margin-hauteurBarreVie);
		self.mana:SetWidth(self:GetWidth()-BarreVie_Margin*2)
		self.mana:SetHeight(hauteurBarreMana);
		self.mana:Hide()
	end
	
	--Créer la barre des soins reçus
	if (not self.barreSoin) then
		self.barreSoin = self:CreateTexture();
		self.barreSoin:SetDrawLayer("OVERLAY")
		self.barreSoin:SetPoint("TOPRIGHT",-BarreVie_Margin,-BarreVie_Margin)
		self.barreSoin:SetTexture(0,0.2,0.8)
	end
	self.barreSoin:SetWidth(0)
	self.barreSoin:SetHeight(self:GetHeight()/4-BarreVie_Margin/2)
	self.soin = 0

	--Créer la barre des dégâts reçus
	if (not self.barreDegats) then
		self.barreDegats = self:CreateTexture();
		self.barreDegats:SetDrawLayer("OVERLAY")
		self.barreDegats:SetTexture(1,0,0)
	end
	self.barreDegats:SetPoint("TOPLEFT",-BarreVie_Margin,-self:GetHeight()/4-BarreVie_Margin)
	self.barreDegats:SetWidth(0)
	self.barreDegats:SetHeight(self:GetHeight()/6)
		
	--Créer les indicateurs de portée
	if (not self.aPortee) then
		self.aPortee = {}
	end		
	
	for i, portee in ipairs(cfgBouton.portee) do
		if (not self.aPortee[i]) then
			self.aPortee[i] = self:CreateTexture();
		end
		self.aPortee[i]:SetTexture("Interface\\AddOns\\Chinon\\Texture\\Portee.tga")
		self.aPortee[i]:SetPoint("BOTTOMLEFT",APortee_Margin+self:GetWidth()/6*(i-1),APortee_Margin);
		self.aPortee[i]:SetHeight(self:GetHeight()/3-APortee_Margin*2);
		self.aPortee[i]:SetWidth(self:GetWidth()/6-APortee_Margin*2);
	end
	
	--Créer zone de debuff en arrière plan du bouton
	if (not self.debuff) then
		self.debuff = {}
	end
	
	local largeurDebuff = (Chinon.apparence.largeurBouton-BarreVie_Margin*2) / #cfgBouton.debuff;
	for i, debuff in ipairs(cfgBouton.debuff) do
		if (not self.debuff[i]) then
			self.debuff[i] = self:CreateTexture();
		end
		self.debuff[i]:SetDrawLayer("BORDER");
		self.debuff[i]:SetPoint("TOPLEFT",largeurDebuff*(i-1)+BarreVie_Margin,-BarreVie_Margin);
		self.debuff[i]:SetHeight(self:GetHeight()-BarreVie_Margin*2);
		self.debuff[i]:SetWidth(largeurDebuff);
		self.debuff[i]:SetTexture(0,0,0);
	end

	--Le nom de la cible (en raid uniquement)
	if ((cfgBouton.nomCible and GetNumRaidMembers()>5) or cfgBouton.menace or cfgBouton.pv) then
		if (not self.prio) then
			self.prio = self:CreateFontString(nil, "OVERLAY");
		end
		self.prio:SetFontObject("GameFontNormal");
		self.prio:SetAllPoints(self);
		if (cfgBouton.nomCible) then
			self.prio:SetText(UnitName(self:GetAttribute("unit")));
		end
		self.prio:SetTextColor(1,1,1);
		self.prio:Show()
	else
		if (self.prio) then
			self.prio:Hide()
		end
	end

	--L'indicateur de rôle
	if cfgBouton.role then
		if not self.role then
			self.role = self:CreateTexture()
		end
		local largeur = Chinon.apparence.hauteurBouton / 3
		if largeur>16 then
			largeur = 16
		end
		self.role:SetHeight(largeur)
		self.role:SetWidth(largeur)
		self.role:SetPoint("BOTTOMRIGHT", largeur/4, -largeur/4)
	end
	
	--Les icônes de buff/debuff
	if (not self.buff) then
		self.buff = {}
	end

	for i=1,#self.buff do
		self:ConfigureBuff(i)
	end
end

function Chinon_Bouton_AddSort(self, sort, cible)
	
	if (sort~=nil) then
		local modifieur = "";
		if (sort.alt) then
			modifieur = modifieur.."alt-"
		end
		if (sort.ctrl) then
			modifieur = modifieur.."ctrl-"
		end
		if (sort.shift) then
			modifieur = modifieur.."shift-"
		end
		self:SetAttribute(modifieur.."type"..sort.bouton,"macro");
		local macro = "";
		if (sort.macro and sort.macro~="") then
			macro = sort.macro
		else
			if (sort.sortRes) then
				local nom = Chinon:GetSpellInfoOrNil(sort.sortRes)
				if (nom) then
					macro = macro.."/cast [dead,target="..cible.."] "..nom.."\n"
				end
			end
			
			if (sort.sortHorsCombat) then
				local nom = Chinon:GetSpellInfoOrNil(sort.sortHorsCombat)
				if (nom) then
					macro = macro.."/cast [nocombat,target="..cible.."] "..nom.."\n"
				end
			end
						
			if (sort.sort) then
				local nom = Chinon:GetSpellInfoOrNil(sort.sort);
				if (nom) then
					macro = macro .. "/cast [target="..cible.."] "..nom;
				end
			end
		end
		self:SetAttribute(modifieur.."macrotext"..sort.bouton,macro);
	end
end

function Chinon_Bouton_OnEnter(self)
	local unit = self:GetAttribute("unit")
	if (UnitName(unit)) then
		Chinon_Debuffs:SetUnit(unit)
	end
end

function Chinon_Bouton_OnLeave(self)
	Chinon_Debuffs:Hide()
end

